﻿using DoubleDeckerBus;
using System.Text;

namespace DoubleDeckerBus
{
    internal class MapsCollection
    {
        
        readonly Dictionary<string, MapWithSetBusesGeneric<IDrawingObject, AbstractMap>> _mapStorages;
       
        public List<string> Keys => _mapStorages.Keys.ToList();
       
        private readonly int _pictureWidth;
        
        private readonly int _pictureHeight;

        private readonly char separatorDict = '|';

        private readonly char separatorData = ';';

        public MapsCollection(int pictureWidth, int pictureHeight)
        {
            _mapStorages = new Dictionary<string, MapWithSetBusesGeneric<IDrawingObject, AbstractMap>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }
       
        public void AddMap(string name, AbstractMap map)
        {
            if (Keys.Contains(name)) return;
            _mapStorages.Add(name, new MapWithSetBusesGeneric<IDrawingObject, AbstractMap>(_pictureWidth, _pictureHeight, map));
        }

        public void DelMap(string name)
        {
            _mapStorages.Remove(name);
        }

        public MapWithSetBusesGeneric<IDrawingObject, AbstractMap> this[string ind]
        {
            get
            {
                _mapStorages.TryGetValue(ind, out var result);
                return result;
            }
        }

        private static void WriteToFile(string text, FileStream stream)
        {
            byte[] info = new UTF8Encoding(true).GetBytes(text);
            stream.Write(info, 0, info.Length);
        }

        public void SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            using (StreamWriter sw = new(filename))
            {
                sw.Write($"MapsCollection{Environment.NewLine}");
                foreach (var storage in _mapStorages)
                {
                    sw.Write($"{storage.Key}{separatorDict}{storage.Value.GetData(separatorDict, separatorData)}{Environment.NewLine}");
                }
            }
        }

        public void LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                throw new FileNotFoundException("Файл не найден");
            }
            string line;
            using (StreamReader sw = new(filename))
            {
                line = sw.ReadLine();
                if (line == null || !line.Contains("MapsCollection"))
                {
                    throw new FileFormatException("Формат данных в файле не совпадает");
                }

                _mapStorages.Clear();
                line = sw.ReadLine();
                while (line != null)
                {
                    var elem = line.Split(separatorDict);
                    AbstractMap map = null;
                    switch (elem[1])
                    {
                        case "SimpleMap":
                            map = new SimpleMap();
                            break;
                        case "WaterMap":
                            map = new WaterMap();
                            break;
                    }
                    _mapStorages.Add(elem[0], new MapWithSetBusesGeneric<IDrawingObject, AbstractMap>(_pictureWidth, _pictureHeight, map));
                    _mapStorages[elem[0]].LoadData(elem[2].Split(separatorData, StringSplitOptions.RemoveEmptyEntries));
                    line = sw.ReadLine();
                }
            }
        }
    }
}
